using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Data.Linq;
using System.Data.Linq.Mapping;

using SqlProc;
using QueryType = SqlProc.SqlTool.QueryType;

namespace SQLTool_Program
{
	public partial class FormMain : Form
	{
		public FormMain()
		{
			InitializeComponent();
		}

		string _cnnString;
		QueryType _qtype;
		string _qtext;

		private void Execute(QueryType qtype, string qtext)
		{
			int rows;
			dataGridView_Result.DataSource = null;
			dataGridView_Result.DataSource = SqlTool.ExecuteQuery(qtext, _cnnString, qtype, out rows);
			if (qtype != QueryType.Table)
			{
				dataGridView_Result.ReadOnly = true;
				dataGridView_Result.AllowUserToAddRows = false;
				dataGridView_Result.AllowUserToDeleteRows = false;
			}
			else
			{
				dataGridView_Result.ReadOnly = false;
				dataGridView_Result.AllowUserToAddRows = true;
				dataGridView_Result.AllowUserToDeleteRows = true;
			}
			this.Text = qtype.ToString() + " : " + qtext.Split(null)[0] + " (" + rows.ToString() + " rows)";
			_qtype = qtype;
			_qtext = qtext;
		}

		private void toolStripButton_Schema_Click(object sender, EventArgs e)
		{
			Execute(QueryType.Schema, "");
		}

		private void toolStripButton_Execute_Click(object sender, EventArgs e)
		{
			Execute(QueryType.SQL, textBox_SQL.Text);
		}

		private void dataGridView_Result_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
		{
			if (_qtype == QueryType.Schema && _qtext == "" && e.ColumnIndex >= 0 && e.RowIndex >= 0)
			{
				if (dataGridView_Result.Columns[e.ColumnIndex].Name == "CollectionName")
				{
					DataGridViewCell cell = dataGridView_Result[e.ColumnIndex, e.RowIndex];
					if (cell.Value is string)
					{
						Execute(QueryType.Schema, (string)cell.Value);
					}
				}
			}
		}

		private void treeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
		{
			treeView.SelectedNode = e.Node;
		}

		private void openToolStripMenuItem_Click(object sender, EventArgs e)
		{
			TreeNode tn = treeView.SelectedNode;
			if (tn != null)
			{
				Execute(QueryType.Table, tn.Parent == null ? tn.Name : tn.Parent.Name);
			}
		}

		private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SqlTool.BuildTree(treeView, _cnnString);
		}

		private void textBox_SQL_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Enter && e.Control)
			{
				e.SuppressKeyPress = true;
				Execute(QueryType.SQL, textBox_SQL.Text);
			}
		}

		private void toolStripButton_Update_Click(object sender, EventArgs e)
		{
			if (_qtype == QueryType.Table)
			{
				//??֪ʲôԭʱInsertUpdate
				dataGridView_Result.EndEdit();
				using (OleDbDataAdapter adapter = new OleDbDataAdapter(_qtext, _cnnString))
				{
					adapter.SelectCommand.CommandType = CommandType.TableDirect;
					OleDbCommandBuilder cb = new OleDbCommandBuilder(adapter);
					cb.QuotePrefix = "[";
					cb.QuoteSuffix = "]";
					adapter.Update((DataTable)dataGridView_Result.DataSource);
				}
				MessageBox.Show("SUCCESS");
			}
		}

		private void toolStripButton_RunScript_Click(object sender, EventArgs e)
		{
			SqlForm.RunScript(_cnnString);
		}

		private void toolStripMenuItem_Open_CNN_Click(object sender, EventArgs e)
		{
			string s = ToolProc.Tools.InputBox("Please input the connection string:", _cnnString);
			if (s != null && s.Trim() != "")
			{
				_cnnString = s;
				SqlTool.BuildTree(treeView, _cnnString);
				dataGridView_Result.DataSource = null;
				this.Text = s.Replace("\r\n", " ");
			}
		}

		private void toolStripMenuItem_Open_MDB_Click(object sender, EventArgs e)
		{
			OpenFileDialog ofd = new OpenFileDialog();
			ofd.Filter = "Access Databases (*.mdb)|*.mdb|All Files (*.*)|*.*";
			if (ofd.ShowDialog() == DialogResult.OK)
			{
				_cnnString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + ofd.FileName + ";";
				SqlTool.BuildTree(treeView, _cnnString);
				dataGridView_Result.DataSource = null;
				this.Text = ofd.FileName;
			}
		}

		private void toolStripMenuItem_Open_SDF_Click(object sender, EventArgs e)
		{
			OpenFileDialog ofd = new OpenFileDialog();
			ofd.Filter = "SQL Server Compact (*.sdf)|*.sdf|All Files (*.*)|*.*";
			if (ofd.ShowDialog() == DialogResult.OK)
			{
				_cnnString = "Provider=Microsoft.SQLSERVER.CE.OLEDB.3.5;Data Source=" + ofd.FileName + ";";
				SqlTool.BuildTree(treeView, _cnnString);
				dataGridView_Result.DataSource = null;
				this.Text = ofd.FileName;
			}
		}

		private void toolStripMenuItem_Open_XLS_Click(object sender, EventArgs e)
		{
			OpenFileDialog ofd = new OpenFileDialog();
			ofd.Filter = "Excel (*.xls)|*.xls|All Files (*.*)|*.*";
			if (ofd.ShowDialog() == DialogResult.OK)
			{
				_cnnString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + ofd.FileName + ";Extended Properties=\"Excel 8.0;HDR=No;IMEX=1\";";
				SqlTool.BuildTree(treeView, _cnnString);
				dataGridView_Result.DataSource = null;
				this.Text = ofd.FileName;
			}
		}

		//Accessݿ(.mdb)ʹADOXMicrosoft ADO Ext, msadox.dll
		private void toolStripMenuItem_Create_MDB_Click(object sender, EventArgs e)
		{
			SaveFileDialog sfd = new SaveFileDialog();
			sfd.Filter = "Access Databases (*.mdb)|*.mdb|All Files (*.*)|*.*";
			if (sfd.ShowDialog() == DialogResult.OK)
			{
				ADOX.CatalogClass cat = new ADOX.CatalogClass();
				string cnn = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + sfd.FileName + ";";
				cat.Create(cnn);
				_cnnString = cnn;
				SqlTool.NonQuery("CREATE TABLE temp (id Text)", _cnnString);
				SqlTool.BuildTree(treeView, _cnnString);
				dataGridView_Result.DataSource = null;
				this.Text = sfd.FileName;
			}
		}

		//SQL Server Compactݿ(.sdf)ʹ"LINQ to SQL"System.Data.Linq, System.Data.Linq.dll
		#region create sdf
#pragma warning disable 0649
		[Table(Name = "temp")]
		class tb_temp
		{
			[Column(IsPrimaryKey = true)]
			public string id;
		}
		class db_empty : DataContext
		{
			public Table<tb_temp> temp;
			public db_empty(string connection) : base(connection) { }
		}
#pragma warning restore 0649
		private void toolStripMenuItem_Create_SDF_Click(object sender, EventArgs e)
		{
			SaveFileDialog sfd = new SaveFileDialog();
			sfd.Filter = "SQL Server Compact Databases (*.sdf)|*.sdf|All Files (*.*)|*.*";
			if (sfd.ShowDialog() == DialogResult.OK)
			{
				db_empty db = new db_empty(sfd.FileName);
				db.CreateDatabase();
				_cnnString = "Provider=Microsoft.SQLSERVER.CE.OLEDB.3.5;Data Source=" + sfd.FileName + ";";
				SqlTool.BuildTree(treeView, _cnnString);
				dataGridView_Result.DataSource = null;
				this.Text = sfd.FileName;
			}
		}
		#endregion

		private void toolStripButton_Test_Click(object sender, EventArgs e)
		{
			int i;
			Form t = new Form();
			TextBox o = new TextBox();
			o.Multiline = true;
			o.Dock = DockStyle.Fill;
			t.Controls.Add(o);
			t.Show();

			//ʹSqlCeʹOleDbsdfļҪò٣һʱࣩ
			//SqlCeConnection֧GetSchema("Tables")ʹSELECT * FROM INFORMATION_SCHEMA.TABLES棩
			//string cnn2 = @"Data Source=..\..\..\MyMoney\old\MyMoney.sdf;Persist Security Info=False;"; //sqlce connection string to sdf file
			//SqlTool.dbFactory = MySqlCeFactory.Instance;
			//i = System.Environment.TickCount;
			//for (int j = 0; j < 200; ++j)
			//{
			//    SqlTool.TextQuery("select * from normal", cnn2);
			//}
			//o.AppendText(((System.Environment.TickCount - i).ToString()) + "\r\n");

			//mdbļܱsdfļҪǳ
			string cnn3 = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=..\..\..\MyMoney\old\MyMoney.mdb;"; //oledb connection string to mdb file
			SqlTool.dbFactory = System.Data.OleDb.OleDbFactory.Instance;
			i = System.Environment.TickCount;
			for (int j = 0; j < 200; ++j)
			{
				SqlTool.TextQuery("select * from normaloutgo", cnn3);
			}
			o.AppendText(((System.Environment.TickCount - i).ToString()) + "\r\n");

			//ʹodbcmdbļܱoledb
			string cnn4 = @"Driver={Microsoft Access Driver (*.mdb)};DBQ=..\..\..\MyMoney\old\MyMoney.mdb;"; //odbc connection string to mdb file
			SqlTool.dbFactory = System.Data.Odbc.OdbcFactory.Instance;
			i = System.Environment.TickCount;
			for (int j = 0; j < 200; ++j)
			{
				SqlTool.TextQuery("select * from normaloutgo", cnn4);
			}
			o.AppendText(((System.Environment.TickCount - i).ToString()) + "\r\n");

			//׼
			string cnn1 = @"Provider=Microsoft.SQLSERVER.CE.OLEDB.3.5;Data Source=..\..\..\MyMoney\old\MyMoney.sdf;"; //oledb connection string to sdf file
			SqlTool.dbFactory = System.Data.OleDb.OleDbFactory.Instance;
			i = System.Environment.TickCount;
			for (int j = 0; j < 200; ++j)
			{
				SqlTool.TextQuery("select * from normal", cnn1);
			}
			o.AppendText(((System.Environment.TickCount - i).ToString()) + "\r\n");

			//ܱȻDbFactoryĻ׼Ժһ
			i = System.Environment.TickCount;
			for (int j = 0; j < 200; ++j)
			{
				SqlTool.TextQuery_OleDb("select * from normal", cnn1);
			}
			o.AppendText(((System.Environment.TickCount - i).ToString()) + "\r\n");

			//ܱȻ׼һ
			i = System.Environment.TickCount;
			for (int j = 0; j < 200; ++j)
			{
				DataTable x = new DataTable();
				x.Load(SqlTool.ReaderQuery("select * from normal", cnn1));
			}
			o.AppendText(((System.Environment.TickCount - i).ToString()) + "\r\n");

		}

	}
}