using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.OleDb;

namespace SQLProc
{
	public enum QueryType { Schema, Table, SQL };
	public static class SQLTool
	{
		public static void BuildTree(string cnnString, System.Windows.Forms.TreeView tv)
		{
			using (OleDbConnection cnn = new OleDbConnection(cnnString))
			{
				cnn.Open();
				Dictionary<string, string> tabledict = new Dictionary<string, string>();
				using (DataTable dtable = cnn.GetSchema("Tables"))
				{
					foreach (DataRow row in dtable.Rows)
					{
						tabledict.Add((string)row["TABLE_NAME"], (string)row["TABLE_TYPE"]);
					}
				}
				SortedList<int, string> typedict = new SortedList<int, string>();
				using (DataTable dtype = cnn.GetSchema("DataTypes"))
				{
					foreach (DataRow row in dtype.Rows)
					{
						short d = (short)row["NativeDataType"];
						if (!typedict.ContainsKey(d))
						{
							typedict.Add(d, (string)row["TypeName"]);
						}
					}
				}
				using (DataTable dcolumn = cnn.GetSchema("Columns"))
				{
					tv.BeginUpdate();
					tv.Nodes.Clear();
					foreach (DataRow row in dcolumn.Select(null, "ORDINAL_POSITION ASC"))
					{
						string table_name = (string)row["TABLE_NAME"];
						string table_type = tabledict[table_name];
						if (table_type == "TABLE" || table_type == "VIEW")
						{
							if (!tv.Nodes.ContainsKey(table_name))
							{
								tv.Nodes.Add(table_name, table_name + " : " + table_type);
							}
							tv.Nodes[table_name].Nodes.Add((string)row["COLUMN_NAME"] + " : " + typedict[(int)row["DATA_TYPE"]]);
						}
					}
					tv.EndUpdate();
				}
			}
		}

		//public static System.Windows.Forms.Form BuildQueryForm(string cnnString)
		//{
		//    System.Windows.Forms.Form f = new System.Windows.Forms.Form();
		//    TextBox textbox = new TextBox();
		//    textbox.Dock = DockStyle.Top;
		//    textbox.Font = new Font("Courier New", 9.75F);
		//    DataGridView datagrid = new DataGridView();
		//    datagrid.Dock = DockStyle.Fill;
		//    textbox.KeyDown += delegate(object _sender, KeyEventArgs _e)
		//    {
		//        if (_e.KeyCode == Keys.Return)
		//        {
		//            _e.SuppressKeyPress = true; //ıڻسʱbeep
		//            int rows;
		//            datagrid.DataSource = NormalIO.ExecuteSQL(textbox.Text, out rows);
		//            form.Text = rows.ToString();
		//        }
		//    };
		//    TreeView tv = new TreeView();
		//    SQLProc.SQLTool.BuildTree(NormalIO._cnnString, tv);
		//    tv.Dock = DockStyle.Left;
		//    form.Controls.Add(tv);
		//    form.Controls.Add(datagrid);
		//    form.Controls.Add(textbox);
		//    form.Show();
		//    textbox.Focus();

		//}

		public static DataTable ExecuteQuery(string cnnString, QueryType qtype, string qtext, out int rows)
		{
			DataTable dt;
			switch (qtype)
			{
				case QueryType.Schema:
					using (OleDbConnection cnn = new OleDbConnection(cnnString))
					{
						cnn.Open();
						dt = qtext == "" ? cnn.GetSchema() : cnn.GetSchema(qtext);
						rows = dt.Rows.Count;
					}
					break;
				case QueryType.Table:
					using (OleDbDataAdapter adapter = new OleDbDataAdapter(qtext, cnnString))
					{
						adapter.SelectCommand.CommandType = CommandType.TableDirect;
						dt = new DataTable();
						rows = adapter.Fill(dt);
					}
					break;
				case QueryType.SQL:
					if (qtext.StartsWith("SELECT", StringComparison.OrdinalIgnoreCase))
					{
						using (OleDbDataAdapter adapter = new OleDbDataAdapter(qtext, cnnString))
						{
							dt = new DataTable();
							rows = adapter.Fill(dt);
						}
					}
					else
					{
						using (OleDbConnection cnn = new OleDbConnection(cnnString))
						{
							cnn.Open();
							OleDbCommand cmd = new OleDbCommand(qtext, cnn);
							rows = cmd.ExecuteNonQuery();
							dt = null;
						}
					}
					break;
				default:
					dt = null;
					rows = 0;
					break;
			}
			return dt;
		}

		public static void ShowQuery(string cnnString, string queryString)
		{
			System.Windows.Forms.Form form = new System.Windows.Forms.Form();
			System.Windows.Forms.DataGridView datagrid = new System.Windows.Forms.DataGridView();
			datagrid.Dock = System.Windows.Forms.DockStyle.Fill;
			int rows;
			datagrid.DataSource = ExecuteQuery(cnnString, QueryType.SQL, queryString, out rows);
			form.Text = rows.ToString();
			datagrid.ReadOnly = true;
			datagrid.AllowUserToAddRows = false;
			datagrid.AllowUserToDeleteRows = false;
			form.Controls.Add(datagrid);
			form.Show();
		}
	}
}
